# C 语言中的错误处理

> 原文：<https://learnetutorials.com/c-programming/error-handling>

在本教程中，您将学习 C 语言提供的某些附加功能，以实现错误处理，尽管 C 语言不提供直接支持。

## 什么是错误处理？

错误处理和调试是编程的重要部分。错误处理是检测和解决干扰正常操作的不必要压力的过程。基本上，错误处理涉及两种不同的活动:

1.  **错误检测**:发现已经发生的错误的过程
2.  **错误恢复**:采取措施纠正错误的过程

在大多数现代语言中，程序员提前预测错误的可能性，他们在程序中加入了一些异常代码，以一次性解决检测和恢复的错误。不幸的是，在 C 例程中，这个特性是不可用的，因此我们最终需要支持错误处理的附加功能

## 什么是 errno？

C 提供的一个处理错误的附加功能是**错误号**，简称为**错误号**。如果任何函数调用失败，这个预定义的全局变量`errno`返回像“-1”或“空”这样的值，向程序员通知错误状态。它的所有位置都写在头文件`error.h`中。为了快速识别，始终建议将变量初始化为“0”。所以在程序启动期间，errno 的值将为零。编译结束后，程序员可以记录返回值并采取必要的步骤。

下表列出了 errno 及其对应的错误类型。

| **误差值** | **错误** |
| one | 不允许操作 |
| Two | 没有这样的文件或目录 |
| three | 没有这样的过程 |
| four | 中断的系统调用 |
| five | 输入输出错误 |
| six | 没有这样的设备或地址 |
| seven | 参数列表太长 |
| eight | Exec 格式错误 |
| nine | 错误的文件编号 |
| Ten | 没有子进程 |
| Eleven | 再试 |
| Twelve | 被遗忘 |
| Thirteen | 权限被拒绝 |

## `perror()`和`strerror()`与 errno 有什么联系

为了显示与`errno`相关的错误信息，在 C 中使用了以下两个功能:

### 1 . perror()

该函数显示传递的字符串，后跟冒号和空格，然后是存储在 errno 中的错误代码的描述。

当我们编写代码来打开或读取文件时，编译器会指示操作系统这样做。但有时如果文件不存在或损坏，它可能会失败。然后库文件`error.h`会在`errno`中保存一个错误代码。使用`perror`，我们可以显示想要的短信，而不仅仅是代码。请看下面的程序:

```c
 #include <stdlib.h>

int main()
{

  FILE *fp;

  fp = fopen("my_file.txt", "r");
  if (fp == NULL)
  {
     printf("Value of errno: %d\n ", errno);
     perror("Message from perror");
   }
} 

```

在这里，编译器将查找文件“my_file.txt”。如果没有找到，文件指针将返回空值，并且 perror 将显示消息“没有这样的文件或目录”。在屏幕上，我们将看到带有`perror()`的传递的字符串，然后是冒号和空格，后面是这条消息。

**输出:**

```c
 Value of errno: 2
Message from perror: No such file or directory 
```

### 2 . strerror()

这是 string.h 头文件中定义的另一个函数，它返回一个指向系统生成的与 errno 相关联的错误消息的指针。检查以下示例，了解这两个函数之间的核心区别。

```c
 #include <stdio.h>
#include <error.h>
#include <string.h>
#include<stdlib.h>

int main()
{

      FILE *fp;

       fp = fopen("my_file.txt", "r");
        if (fp == NULL)
         {
            printf("Value of errno: %d\n ", errno);
            perror("Message from perror");
            printf("The error message by strerror is : %s\n", strerror(errno));
        }
        else{
            fclose(fp);
        }
} 

```

在这里，函数 perror 和 strerror 都将打印相同的消息“没有这样的文件或目录”。不同的是，前者表示 errno 的文本格式，而后者通过传递错误号作为函数的参数来实现。

```c
 Value of errno: 2
Message from perror:No such file or directory 
The error message by strerror is : No such file or directory 
```

## 除以零误差

我们经常遇到这种类型的错误，尤其是涉及复杂的数学计算。在为计算器或银行软件编写代码时，我们可能会忘记用户可能会将除数的数据输入为零。我们知道，任何数除以零都会得到无穷大，所以如果发生这种情况，程序会标记一个错误。

因此，最佳实践是在单独的代码块中路由除数为零的所有情况。这将标记一个适当的错误消息，用户将很容易理解出了什么问题。在下面的程序中，我们将尝试用 1 除以 0，并使用 strerror 函数标记错误。

```c
 #include <stdio.h>
#include <stdlib.h>
main()
 {
        int a = 1;
        int b = 0;
        int c;
        if( b == 0)
        {
                 fprintf(stderr, " Zero can not be a divisor...\n");
                 exit(-1);
         }

    c = a / b;
   fprintf(stderr, "The result of the division is %d\n", c );
    exit(0);
} 

```

**输出:**

```c
 Zero cannot be a divisor… 
```

## 程序退出状态

一般来说，一个项目有两种可能的结果，成功或失败。在程序员中，当成功运行时，返回 exit 值“0”是一种很好的做法。否则，返回“-1”作为否定结果的表示。当输入数据不正确时，执行其余代码是没有用的。所以最好带着消息退出程序。

这是一个部门计划:

```c
 #include <stdio.h>
#include <stdlib.h>
main()
 {
    int a, b, c;
    printf(" Enter the dividend");
    scanf("%d",&a);
    printf ("\n Enter the divisor");
    scanf("%d",&b);
    if( b== 0)
    {
        fprintf(stderr, "\nZero cannot be a divisor! ");
        exit(EXIT_FAILURE);
    }
    c=a/b;
    fprintf(stderr, " \nTHE RESULT IS : %d", c );
    exit(EXIT_SUCCESS);
} 

```

如果输入零作为除数，程序将退出，并显示已定义的错误消息“零不能是除数！”。否则，它将显示正确的商。

**输出:**

```c
Enter the dividend  40
Enter the divisor 2
THE RESULT IS : 20
                OR
Enter the dividend 4
Enter the divisor 0
Zero cannot be a divisor! 
```